@@ -1,6 +1,6 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
  | Copyright (C) 2011, The Roundcube Dev Team                            |
@@ -18,7 +18,6 @@
  +-----------------------------------------------------------------------+
 */
 
-
 /**
  * Interface class for accessing Roundcube cache
  *
@@ -42,6 +41,8 @@ class rcube_cache
     private $ttl;
     private $packed;
     private $index;
+    private $debug;
+    private $index_changed = false;
     private $cache         = array();
     private $cache_changes = array();
     private $cache_sums    = array();
@@ -65,12 +66,14 @@ class rcube_cache
         $type  = strtolower($type);
 
         if ($type == 'memcache') {
-            $this->type = 'memcache';
-            $this->db   = $rcube->get_memcache();
+            $this->type  = 'memcache';
+            $this->db    = $rcube->get_memcache();
+            $this->debug = $rcube->config->get('memcache_debug');
         }
         else if ($type == 'apc') {
-            $this->type = 'apc';
-            $this->db   = function_exists('apc_exists'); // APC 3.1.4 required
+            $this->type  = 'apc';
+            $this->db    = function_exists('apc_exists'); // APC 3.1.4 required
+            $this->debug = $rcube->config->get('apc_debug');
         }
         else {
             $this->type  = 'db';
@@ -88,7 +91,6 @@ class rcube_cache
         $this->prefix    = $prefix;
     }
 
-
     /**
      * Returns cached value.
      *
@@ -105,7 +107,6 @@ class rcube_cache
         return $this->cache[$key];
     }
 
-
     /**
      * Sets (add/update) value in cache.
      *
@@ -115,11 +116,9 @@ class rcube_cache
     function set($key, $data)
     {
         $this->cache[$key]         = $data;
-        $this->cache_changed       = true;
         $this->cache_changes[$key] = true;
     }
 
-
     /**
      * Returns cached value without storing it in internal memory.
      *
@@ -136,7 +135,6 @@ class rcube_cache
         return $this->read_record($key, true);
     }
 
-
     /**
      * Sets (add/update) value in cache and immediately saves
      * it in the backend, no internal memory will be used.
@@ -151,7 +149,6 @@ class rcube_cache
         return $this->write_record($key, $this->serialize($data));
     }
 
-
     /**
      * Clears the cache.
      *
@@ -164,7 +161,6 @@ class rcube_cache
         // Remove all keys
         if ($key === null) {
             $this->cache         = array();
-            $this->cache_changed = true;
             $this->cache_changes = array();
             $this->cache_sums    = array();
         }
@@ -189,7 +185,6 @@ class rcube_cache
         $this->remove_record($key, $prefix_mode);
     }
 
-
     /**
      * Remove cache records older than ttl
      */
@@ -206,7 +201,6 @@ class rcube_cache
         }
     }
 
-
     /**
      * Remove expired records of all caches
      */
@@ -218,16 +212,11 @@ class rcube_cache
         $db->query("DELETE FROM " . $db->table_name('cache', true) . " WHERE `expires` < " . $db->now());
     }
 
-
     /**
      * Writes the cache back to the DB.
      */
     function close()
     {
-        if (!$this->cache_changed) {
-            return;
-        }
-
         foreach ($this->cache as $key => $data) {
             // The key has been used
             if ($this->cache_changes[$key]) {
@@ -241,13 +230,14 @@ class rcube_cache
             }
         }
 
-        $this->write_index();
+        if ($this->index_changed) {
+            $this->write_index();
+        }
 
         // reset internal cache index, thanks to this we can force index reload
         $this->index = null;
     }
 
-
     /**
      * Reads cache entry.
      *
@@ -271,11 +261,19 @@ class rcube_cache
                 // to have data in consistent state. Keeping the index consistent
                 // is needed for keys delete operation when we delete all keys or by prefix.
             }
-            else if ($this->type == 'memcache') {
-                $data = $this->db->get($this->ckey($key));
-            }
-            else if ($this->type == 'apc') {
-                $data = apc_fetch($this->ckey($key));
+            else {
+                $ckey = $this->ckey($key);
+
+                if ($this->type == 'memcache') {
+                    $data = $this->db->get($ckey);
+                }
+                else if ($this->type == 'apc') {
+                    $data = apc_fetch($ckey);
+                }
+
+                if ($this->debug) {
+                    $this->debug('get', $ckey, $data);
+                }
             }
 
             if ($data) {
@@ -325,7 +323,6 @@ class rcube_cache
         return $this->cache[$key];
     }
 
-
     /**
      * Writes single cache record into DB.
      *
@@ -350,9 +347,16 @@ class rcube_cache
             $result = $this->add_record($this->ckey($key), $data);
 
             // make sure index will be updated
-            if ($result && !array_key_exists($key, $this->cache_sums)) {
-                $this->cache_changed    = true;
-                $this->cache_sums[$key] = true;
+            if ($result) {
+                if (!array_key_exists($key, $this->cache_sums)) {
+                    $this->cache_sums[$key] = true;
+                }
+
+                $this->load_index();
+
+                if (!$this->index_changed && !in_array($key, $this->index)) {
+                    $this->index_changed = true;
+                }
             }
 
             return $result;
@@ -396,7 +400,6 @@ class rcube_cache
         return $this->db->affected_rows($result);
     }
 
-
     /**
      * Deletes the cache record(s).
      *
@@ -416,23 +419,30 @@ class rcube_cache
             // Remove all keys
             if ($key === null) {
                 foreach ($this->index as $key) {
-                    $this->delete_record($key, false);
+                    $this->delete_record($this->ckey($key));
                 }
+
                 $this->index = array();
             }
             // Remove keys by name prefix
             else if ($prefix_mode) {
-                foreach ($this->index as $k) {
+                foreach ($this->index as $idx => $k) {
                     if (strpos($k, $key) === 0) {
-                        $this->delete_record($k);
+                        $this->delete_record($this->ckey($k));
+                        unset($this->index[$idx]);
                     }
                 }
             }
             // Remove one key by name
             else {
-                $this->delete_record($key);
+                $this->delete_record($this->ckey($key));
+                if (($idx = array_search($key, $this->index)) !== false) {
+                    unset($this->index[$idx]);
+                }
             }
 
+            $this->index_changed = true;
+
             return;
         }
 
@@ -454,7 +464,6 @@ class rcube_cache
             $this->userid);
     }
 
-
     /**
      * Adds entry into memcache/apc DB.
      *
@@ -467,50 +476,56 @@ class rcube_cache
     {
         if ($this->type == 'memcache') {
             $result = $this->db->replace($key, $data, MEMCACHE_COMPRESSED, $this->ttl);
-            if (!$result)
+
+            if (!$result) {
                 $result = $this->db->set($key, $data, MEMCACHE_COMPRESSED, $this->ttl);
+            }
         }
         else if ($this->type == 'apc') {
-            if (apc_exists($key))
+            if (apc_exists($key)) {
                 apc_delete($key);
+            }
+
             $result = apc_store($key, $data, $this->ttl);
         }
 
+        if ($this->debug) {
+            $this->debug('set', $key, $data, $result);
+        }
+
         return $result;
     }
 
-
     /**
      * Deletes entry from memcache/apc DB.
+     *
+     * @param string $key Cache key name
+     *
+     * @param boolean True on success, False on failure
      */
-    private function delete_record($key, $index=true)
+    private function delete_record($key)
     {
         if ($this->type == 'memcache') {
             // #1488592: use 2nd argument
-            $this->db->delete($this->ckey($key), 0);
+            $result = $this->db->delete($key, 0);
         }
         else {
-            apc_delete($this->ckey($key));
+            $result = apc_delete($key);
         }
 
-        if ($index) {
-            if (($idx = array_search($key, $this->index)) !== false) {
-                unset($this->index[$idx]);
-            }
+        if ($this->debug) {
+            $this->debug('delete', $key, null, $result);
         }
+
+        return $result;
     }
 
-
     /**
      * Writes the index entry into memcache/apc DB.
      */
     private function write_index()
     {
-        if (!$this->db) {
-            return;
-        }
-
-        if ($this->type == 'db') {
+        if (!$this->db || $this->type == 'db') {
             return;
         }
 
@@ -534,13 +549,12 @@ class rcube_cache
         $this->add_record($this->ikey(), $data);
     }
 
-
     /**
      * Gets the index entry from memcache/apc DB.
      */
     private function load_index()
     {
-        if (!$this->db) {
+        if (!$this->db || $this->type == 'db') {
             return;
         }
 
@@ -549,6 +563,7 @@ class rcube_cache
         }
 
         $index_key = $this->ikey();
+
         if ($this->type == 'memcache') {
             $data = $this->db->get($index_key);
         }
@@ -556,10 +571,13 @@ class rcube_cache
             $data = apc_fetch($index_key);
         }
 
+        if ($this->debug) {
+            $this->debug('get', $index_key, $data);
+        }
+
         $this->index = $data ? unserialize($data) : array();
     }
 
-
     /**
      * Creates per-user cache key name (for memcache and apc)
      *
@@ -572,7 +590,6 @@ class rcube_cache
         return sprintf('%d:%s:%s', $this->userid, $this->prefix, $key);
     }
 
-
     /**
      * Creates per-user index cache key name (for memcache and apc)
      *
@@ -635,4 +652,18 @@ class rcube_cache
 
         return $this->max_packet;
     }
+
+    /**
+     * Write memcache/apc debug info to the log
+     */
+    private function debug($type, $key, $data = null, $result = null)
+    {
+        $line = strtoupper($type) . ' ' . $key;
+
+        if ($data !== null) {
+            $line .= ' ' . ($this->packed ? $data : serialize($data));
+        }
+
+        rcube::debug($this->type, $line, $result);
+    }
 }
